﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;
using UnityEngine;

namespace UPP.Engine
{
    public static class Serializer
    {
        public static T FromString<T>(this string toDeserialize)
        {
            XmlSerializer xmlSerializer = new XmlSerializer(typeof(T));
            StringReader textReader = new StringReader(toDeserialize);
            return (T)xmlSerializer.Deserialize(textReader);
        }

        public static string ToString<T>(this T toSerialize)
        {
            XmlSerializer xmlSerializer = new XmlSerializer(typeof(T));
            StringWriter textWriter = new StringWriter();
            xmlSerializer.Serialize(textWriter, toSerialize);
            return textWriter.ToString();
        }

        public static object FromString(this string toDeserialize, string type)
        {
            Type t = Type.GetType(type);
            if (t == null)
            {
                Debug.LogError("Not a valid type: " + type + "; Could not perform Serializer.FromString().");
                return null;
            }

            XmlSerializer xmlSerializer = new XmlSerializer(t);
            StringReader textReader = new StringReader(toDeserialize);
            return (object)xmlSerializer.Deserialize(textReader);
        }

        public static string ToString(object toSerialize, string type)
        {
            Type t = Type.GetType(type);
            if (t == null)
            {
                Debug.LogError("Not a valid type: " + type + "; Could not perform Serializer.ToString().");
                return null;
            }

            XmlSerializer xmlSerializer = new XmlSerializer(t);
            StringWriter textWriter = new StringWriter();
            xmlSerializer.Serialize(textWriter, toSerialize);
            return textWriter.ToString();
        }


        public static string EncapsulateStrings(params string[] values)
        {
            return ToString<string[]>(values);
        }

        public static string[] DecapsulateString(string values)
        {
            return FromString<string[]>(values);
        }
    }
}