﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace UPP {

    public class Region {

        private Size oldSize;
        public Size Size { get { return oldSize; } set { SetSize(value); } }

        private Position oldPosition;
        public Position Position { get { return oldPosition; } set { SetPosition(value); } }

        public delegate void RegionChangedHandler(float oldX, float oldY, float oldWidth, float oldHeight, float newX, float newY, float newWidth, float newHeight);
        public event RegionChangedHandler RegionChanged;

        private void SetPosition(Position newPos)
        {
            if (newPos.X != oldPosition.X || newPos.Y != oldPosition.Y)
            {
                Position back = new Position(oldPosition.X, oldPosition.Y);
                oldPosition = newPos;

                if (RegionChanged != null)
                    RegionChanged(back.X, back.Y, oldSize.Width, oldSize.Height, newPos.X, newPos.Y, oldSize.Width, oldSize.Height);

                newPos.PositionChanged += Position_PositionChanged;
            }
        }

        private void SetSize(Size newSize)
        {
            if (newSize.Width != oldSize.Width || newSize.Height != oldSize.Height)
            {
                Size back = new Size(oldSize.Width, oldSize.Height);
                oldSize = newSize;

                if (RegionChanged != null)
                    RegionChanged(oldPosition.X, oldPosition.Y, back.Width, back.Height, oldPosition.X, oldPosition.Y, newSize.Width, newSize.Height);

                newSize.SizeChanged += Size_SizeChanged;
            }
        }

        private void Initialize()
        {
            this.Size.SizeChanged += Size_SizeChanged;
            this.Position.PositionChanged += Position_PositionChanged;
        }

        private void Position_PositionChanged(float oldX, float oldY, float newX, float newY)
        {
            if (RegionChanged != null)
                RegionChanged(oldX, oldY, oldSize.Width, oldSize.Height, newX, newY, oldSize.Width, oldSize.Height);
        }

        private void Size_SizeChanged(float oldWidth, float oldHeight, float newWidth, float newHeight)
        {
            if (RegionChanged != null)
                RegionChanged(oldPosition.X, oldPosition.Y, oldWidth, oldHeight, oldPosition.X, oldPosition.Y, newWidth, newHeight);
        }

        public Region()
        {
            oldSize = new Size();
            oldPosition = new Position();
            Initialize();
        }

        public Region(Position position, Size size)
        {
            oldPosition = position;
            oldSize = size;
            Initialize();
        }

        public Region(float x, float y, float width, float height)
        {
            oldSize = new Size(width, height);
            oldPosition = new Position(x, y);
            Initialize();
        }

        public override string ToString()
        {
            return this.Position.ToString() + " " + this.Size.ToString();
        }
    }
}