﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UPP.OGUI;
using UnityEditor;
using System;

namespace UPP.OGUI
{

    public class Button : Inline
    {

        private string _text = "";
        public string Text { get { return _text; } set { ChangeText(value); } }

        public delegate void OnClickHandler(object sender);
        public event OnClickHandler OnClick;

        private void Init(string text)
        {
            _text = text;
            this.InheritedContent = DrawInherited;
            SetSizeFromContent();
        }

        public Button()
        {
            Init(this.Name);
        }

        public Button(string text)
        {
            Init(text);
        }

        private void DrawInherited()
        {
            if (AutoSize)
            {
                if (GUILayout.Button(_text, GetCurrentStyle())
                    && OnClick != null)
                    OnClick(this);
            }
            else
            {
                if (GUILayout.Button(_text, GetCurrentStyle(),
                        GUILayout.Width(this.Size.Width - GetCurrentStyle().padding.left - GetCurrentStyle().padding.right),
                        GUILayout.Height(this.Size.Height - GetCurrentStyle().padding.top - GetCurrentStyle().padding.bottom)
                        )
                        && OnClick != null
                    )
                    OnClick(this);
            }
        }

        private void ChangeText(string newVal)
        {
            if (newVal != _text)
            {
                _text = newVal;
                if (AutoSize)
                    SetSizeFromContent();
            }
        }

        public override GUIContent GetContent()
        {
            return new GUIContent(_text);
        }

        public override GUIStyle DefaultStyle()
        {
            GUIStyle skin = new GUIStyle(GUI.skin.button);
            skin.margin = new RectOffset(0, 0, 0, 0);
            return new GUIStyle(skin);
        }
    }


}