﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace UPP.OGUI
{
    public class DialogConfirmPanel : Panel
    {

        private Window window = null;

        public class ConfirmFeedback
        {
            private bool _close = true;
            public bool Close { get { return _close; } set { _close = value; } }
            public ConfirmFeedback() { _close = true; }
            public ConfirmFeedback(bool close) { _close = close; }
        }

        public delegate void OnConfirmHandler(ConfirmFeedback f);
        public event OnConfirmHandler OnConfirm;

        private void Prepare(Rect position, string ok, string storno)
        {
            Dock = DockStyle.Bottom;
            Size = new Size(position.width, 28);
            BackgroundColor = ColorSchemes.BackgroundColorLighter;
            BorderColor = ColorSchemes.BorderColor;

            float off = 0;
            if (storno != null)
            {
                Button b2 = new Button(storno);
                b2.Position = new Position(
                    position.width - b2.Size.Width - 5, 5);
                b2.Parent = this;
                b2.Anchor = new Anchor(true, true, true, false);
                b2.OnClick += B2_OnClick;
                off = b2.Size.Width + 3;
            }

            if (ok != null)
            {
                Button b1 = new Button(ok);
                b1.Position = new Position(
                    position.width - b1.Size.Width - 5 - off, 5);
                b1.Anchor = new Anchor(true, true, true, false);
                b1.Parent = this;
                b1.OnClick += B1_OnClick;
            }
        }

        private void B1_OnClick(object sender)
        {
            // Save button.
            window.Result = OGUI.DialogResult.OK;
            ConfirmFeedback f = new ConfirmFeedback(true);
            if (OnConfirm != null)
                OnConfirm(f);
            if (f.Close)
                window.Close();
        }

        private void B2_OnClick(object sender)
        {
            // Cancel button.
            window.Result = OGUI.DialogResult.Cancel;
            window.Close();
        }

        private void Init(Window window, Rect position, string ok, string cancel)
        {
            this.window = window;
            Prepare(position, ok, cancel);
        }

        public DialogConfirmPanel(Window window, Rect position)
        {
            Init(window, position, "OK", "Cancel");
        }

        public DialogConfirmPanel(Window window, Rect position, string ok, string cancel)
        {
            Init(window, position, ok, cancel);
        }


    }
}