﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace UPP.OGUI
{

    public class MousePanel : Panel
    {

        private void Init()
        {
            this.InheritedContent = DrawInherited;
        }

        public MousePanel() : base() { Init(); }
        public MousePanel(Control parent) : base(parent) { Init(); }

        public enum MouseButton
        {
            Left, Right, Middle, Other
        }

        public delegate void MouseDown(MouseButton button, Event e);
        public event MouseDown OnMouseDown;

        public delegate void MouseUp(Event e);
        public event MouseUp OnMouseUp;

        public delegate void MouseDrag(Event e);
        public event MouseDrag OnMouseDrag;

        private bool _isMouseDown = false;
        public bool IsMouseDown { get { return _isMouseDown; } }

        private void DrawInherited()
        {
            Event e = Event.current;

            if (e.isMouse || _isMouseDown)
            {

                Rect rec = new Rect(0, 0, this.Size.Width, this.Size.Height);

                if (rec.Contains(e.mousePosition) || _isMouseDown)
                {
                    if (e.type == EventType.MouseDown)
                    {
                        MouseButton btn = MouseButton.Other;

                        if (e.button == 0) btn = MouseButton.Left;
                        if (e.button == 1) btn = MouseButton.Right;
                        if (e.button == 2) btn = MouseButton.Middle;

                        if (OnMouseDown != null)
                            OnMouseDown(btn, e);

                        _isMouseDown = true;
                    }

                    if ((e.type == EventType.MouseUp || e.type == EventType.Ignore) && _isMouseDown)
                    {
                        if (OnMouseUp != null)
                            OnMouseUp(e);

                        _isMouseDown = false;
                    }

                    if ((e.type == EventType.MouseMove || e.type == EventType.MouseDrag) && _isMouseDown)
                    {
                        if (OnMouseDrag != null)
                            OnMouseDrag(e);

                    }
                }
            }
        }
    }

}