﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UPP.OGUI;
using UnityEditor;
using System;
using System.Text.RegularExpressions;

namespace UPP.OGUI
{

    public class TextBox : Inline
    {

        private string _value = "";
        public string Value { get { return _value; } set { ChangeValue(value); } }

        private int _maxLength = -1;
        public int MaxLength { get { return _maxLength; } set { SetMaxLength(value); } }

        public delegate void OnChangeHandler(object sender);
        public event OnChangeHandler OnChange;

        private string _mask = "";
        public string Mask { get { return _mask; } set { _mask = value; ChangeValue(_value); } }

        private bool _readOnly = false;
        public bool ReadOnly { get { return _readOnly; } set { _readOnly = value; } }

        Vector2 scroll = new Vector2(0, Mathf.Infinity);

        public void ScrollToEnd()
        {
            scroll = new Vector2(0, Mathf.Infinity);
        }

        private void SetMaxLength(int max)
        {
            if (max < -1)
                max = -1;
            _maxLength = max;
            if (_maxLength > -1)
                _value = _value.Substring(0, _maxLength);
        }

        private void Init(string value, int maxLength)
        {
            _maxLength = maxLength;
            _value = value;
            this.InheritedContent = DrawInherited;
            SetSizeFromContent();
            this.AutoSize = false;
            this.Size = new UPP.Size(150, GetCurrentStyle().CalcHeight(new GUIContent(), 150)
                + GetCurrentStyle().padding.top + GetCurrentStyle().padding.bottom);
        }

        public TextBox()
        {
            Init("", -1);
        }

        public TextBox(string value)
        {
            Init(value, -1);
        }

        public TextBox(int maxLength)
        {
            Init("", maxLength);
        }

        public TextBox(string value, int maxLength)
        {
            Init(value, maxLength);
        }

        private void DrawInherited()
        {
            if (AutoSize)
            {
                string s = "";
                s = GUILayout.TextField(_value, GetCurrentStyle());

                if (s != _value && !ReadOnly)
                    this.Value = s; // Also will trigger the event.
            }
            else
            {
                string s = "";
                scroll = GUILayout.BeginScrollView(scroll);
                float neededHeight = GetCurrentStyle().CalcHeight(GetContent(), Size.Width);
                float realHeight = this.Size.Height - GetCurrentStyle().padding.top - GetCurrentStyle().padding.bottom;
                float height = (neededHeight < realHeight) ? realHeight : neededHeight;
                s = GUILayout.TextArea(_value, GetCurrentStyle(),
                    GUILayout.Width(this.Size.Width - GetCurrentStyle().padding.left - GetCurrentStyle().padding.right),
                    GUILayout.Height(height)
                    );
                GUILayout.EndScrollView();

                if (s != _value && !ReadOnly)
                    this.Value = s; // Also will trigger the event.
            }
        }

        private void ChangeValue(string newVal)
        {
            if (newVal != _value)
            {

                if (_maxLength > -1 && newVal.Length > _maxLength)
                    newVal = newVal.Substring(0, _maxLength);

                if (_mask != "" && newVal != "")
                {
                    Regex r = new Regex(_mask);
                    if (!r.IsMatch(newVal))
                    {
                        EditorUtility.DisplayDialog("Invalid input",
                            "That is not a valid input.\n\nExpected format (RegEx match):\n" +
                            _mask + "\n\nYour change was reverted.", "OK");
                        (this.WindowParent as Window).Focus();
                        return;
                    }
                }

                _value = newVal;

                if (AutoSize)
                    SetSizeFromContent();

                if (OnChange != null)
                    OnChange(this);

            }
        }

        public override GUIContent GetContent()
        {
            return new GUIContent(_value);
        }

        public override GUIStyle DefaultStyle()
        {
            GUIStyle skin = new GUIStyle(GUI.skin.textField);
            skin.margin = new RectOffset(4, 0, 1, 0);
            skin.padding = new RectOffset(5, 5, 1, 1);
            return new GUIStyle(skin);
        }
    }

}