﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;

namespace UPP.OGUI
{
    public abstract class Window : EditorWindow
    {
        private List<DiagramEntity> subwindowsDiagramEntity = new List<DiagramEntity>();

        public DialogResult Result = DialogResult.Cancel;

        private Window _modalChild = null;
        public Window ModalChild { get { return _modalChild; } set { SetModalChild(value); } }

        private Window _modalParent = null;
        public Window ModalParent { get { return _modalParent; } set { SetModalParent(value); } }

        private Panel canvas = null;
        public Panel Canvas { get { return canvas; } }

        private Panel modalPanel = null;

        private bool autoParent = true;
        public bool FillEmptyParents { get { return autoParent; } set { autoParent = value; } }

        public delegate void OnModalReturnHandler(DialogResult result, Type childType, object child);
        public event OnModalReturnHandler OnModalReturn;

        private void SetModalChild(Window w)
        {
            _modalChild = w;
            if (w != null && w.ModalParent != this)
                w.ModalParent = this;
        }

        private void SetModalParent(Window w)
        {
            _modalParent = w;
            if (w != null && w.ModalChild != this)
                w.ModalChild = this;
        }

        public void OnDestroy()
        {
            if (ModalParent != null)
            {
                ModalParent.ModalChild = null;
                ModalParent.Focus();

                if (ModalParent.OnModalReturn != null)
                    ModalParent.OnModalReturn(this.Result, this.GetType(), this);
            }
        }

        private void OnGUI()
        {
            if (canvas == null)
            {
                canvas = new Panel();
                canvas.WindowParent = this;
                canvas.Size = new Size(position.width, position.height); 
                canvas.Dock = DockStyle.Fill;

                modalPanel = new Panel();
                //  modalPanel.Parent = canvas;
                modalPanel.WindowParent = this;
                modalPanel.BackgroundColor = new Color(0, 0, 0, 0.5f);

                WindowObjectRegister.ClearHistory();
                canvas.PreparingSize = true;
                PrepareGUI();
                canvas.Size = new Size(position.width, position.height);
                canvas.PreparingSize = false;
                if (FillEmptyParents)
                    FillParents();
                WindowObjectRegister.ClearHistory();
            }

            BeforeDraw();

            modalPanel.Visible = (_modalChild != null);
            if (modalPanel.Visible)
            {
                Region r = new Region(0, 0, position.width, position.height);
                modalPanel.Size = new Size(position.width, position.height);   
                canvas.Draw(ref r);
                modalPanel.Draw(ref r);
            }
            else
            {
                Region r = new Region(0, 0, position.width, position.height);
                canvas.Draw(ref r);
            }

            AfterDraw();
        }

        private void OnFocus()
        {
            if (ModalChild != null)
                ModalChild.Focus();
        }

        private void FillParents()
        {
            List<Container> l = WindowObjectRegister.GetLastBatch();
            foreach (Container o in l)
            {
                Control c = o as OGUI.Control;
                if (c.Parent == null)
                    c.Parent = Canvas;
            }
        }

        public void RegisterDiagramEntity(DiagramEntity e)
        {
            if (!subwindowsDiagramEntity.Contains(e))
                subwindowsDiagramEntity.Add(e);
        }

        protected virtual void BeforeDraw() { }
        protected virtual void AfterDraw() { }
        protected abstract void PrepareGUI();

    }
}