﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;
using UPP.OGUI;

namespace UPP.Engine
{
    public class ClassEditorEdit : OGUI.Window
    {
        private const int width = 300;
        private const int height = 175;

        public ClassRecord ClassRec = null;

        public enum EditorEditMode
        {
            Create = 0,
            Edit = 1
        }

        public EditorEditMode Mode = EditorEditMode.Create;
        private string preSetParent = "";
        public string PreSetParent { get { return preSetParent; } set { preSetParent = value; } }

        TextBox tbMachineName;
        TextBox tbName;
        ComboBox cbParent;
        TextBox tbDescription;
        CheckBox chbAbstract;

        private void fillParents()
        {
            cbParent.Clear();
            foreach(ClassDBRecord rec in ClassManager.DB.database)
            {
                cbParent.Add(rec.machineName, rec.className);
            }
        }

        protected override void PrepareGUI()
        {
            if (ClassRec == null && Mode == EditorEditMode.Edit)
            {
                Debug.LogError("ClassRec not set while opening ClassEditorEdit in Edit mode.");
                this.Close();
            }

            position = new Rect(position.x, position.y, width, height);
            minSize = new Vector2(width, height);

            string save = (Mode == EditorEditMode.Create) ? "Create" : "Save";
            DialogConfirmPanel p = new DialogConfirmPanel(this, position, save, "Cancel");
            p.OnConfirm += P_OnConfirm;

            Label l1 = new Label("Readable name:");
            l1.Position = new Position(10, 10);

            Label l2 = new Label("Machine name:");
            l2.Position = new Position(10, 35);

            Label l3 = new Label("Parent:");
            l3.Position = new Position(10, 60);

            Label l5 = new Label("Is abstract:");
            l5.Position = new Position(10, 85);

            Label l4 = new Label("Description:");
            l4.Position = new Position(10, 110);

            tbName = new TextBox(31);
            tbName.Anchor = new Anchor(true, true, false, true);
            tbName.Position = new Position(120, 10);
            tbName.Size.Width = 176;
            if (Mode == EditorEditMode.Edit && ClassRec != null)
                tbName.Value = ClassRec.className;

            tbMachineName = new TextBox(31);
            tbMachineName.Anchor = new Anchor(true, true, false, true);
            tbMachineName.Position = new Position(120, 35);
            tbMachineName.Size.Width = 176;
            tbMachineName.Mask = MaskTemplates.MachineName;
            if (Mode == EditorEditMode.Edit && ClassRec != null)
                tbMachineName.Value = ClassRec.machineName;

            cbParent = new ComboBox("<No Parent>");
            cbParent.Anchor = new Anchor(true, true, false, true);
            cbParent.Position = new Position(120, 60);
            cbParent.Size.Width = 176;
            fillParents();
            if (preSetParent != "")
                cbParent.SelectedKey = preSetParent;
            if (Mode == EditorEditMode.Edit && ClassRec != null && ClassRec.machineParent != "")
                cbParent.SelectedKey = ClassRec.machineParent;

            chbAbstract = new CheckBox("Yes");
            chbAbstract.Size.Width = 176;
            chbAbstract.Anchor = new Anchor(true, true, false, true);
            chbAbstract.Position = new Position(120, 85);
            if (Mode == EditorEditMode.Edit && ClassRec != null && ClassRec.isAbstract)
                chbAbstract.Checked = true;

            tbDescription = new TextBox(255);
            tbDescription.Anchor = new Anchor(true, true, false, true);
            tbDescription.Position = new Position(120, 110);
            tbDescription.Size.Width = 176;
            if (Mode == EditorEditMode.Edit && ClassRec != null)
                tbDescription.Value = ClassRec.description;
        }

        private void P_OnConfirm(DialogConfirmPanel.ConfirmFeedback f)
        {
            if (tbMachineName.Value == "" || tbName.Value == "")
            {
                EditorUtility.DisplayDialog("Invalid name", "Class name or machine name is invalid.\nPlease choose another name.", "OK");
                f.Close = false;
                return;
            }

            if (Mode == EditorEditMode.Create)
            {
                string par = "";
                if (cbParent.SelectedIndex != -1)
                    par = cbParent.SelectedKey;
                if (!ClassManager.Create(tbName.Value, tbMachineName.Value, par, tbDescription.Value, chbAbstract.Checked))
                    f.Close = false;
            }

            if (Mode == EditorEditMode.Edit)
            {
                string par = "";
                if (cbParent.SelectedIndex != -1)
                    par = cbParent.SelectedKey;
                if (!ClassManager.Edit(ClassRec, tbName.Value, tbMachineName.Value, par, tbDescription.Value, chbAbstract.Checked))
                    f.Close = false;
            }
        }
    }

}