﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;
using UPP.OGUI;

namespace UPP.Engine
{
    public class TemplateEditorEdit : Window
    {
        private const int width = 300;
        private const int height = 115;

        public TemplateRecord TemplateRec = null;
        public string ParentClass = "";
        public string PreCategory = "";

        TextBox tbMachineName;
        TextBox tbName;
        TextBox tbCategory;

        public enum EditorEditMode
        {
            Create = 0,
            Edit = 1
        }
        public EditorEditMode Mode = EditorEditMode.Create;

        protected override void PrepareGUI()
        {
            if (TemplateRec == null && Mode == EditorEditMode.Edit)
            {
                Debug.LogError("TemplateRec not set while opening TemplateEditorEdit in Edit mode.");
                this.Close();
            }
            position = new Rect(position.x, position.y, width, height);
            minSize = new Vector2(width, height);

            string save = (Mode == EditorEditMode.Create) ? "Create" : "Save";
            DialogConfirmPanel p = new DialogConfirmPanel(this, position, save, "Cancel");
            p.OnConfirm += P_OnConfirm;

            Label l1 = new Label("Readable name:");
            l1.Position = new Position(10, 10);

            Label l2 = new Label("Machine name:");
            l2.Position = new Position(10, 35);

            Label l3 = new Label("Category:");
            l3.Position = new Position(10, 60);

            tbName = new TextBox(31);
            tbName.Anchor = new Anchor(true, true, false, true);
            tbName.Position = new Position(120, 10);
            tbName.Size.Width = 176;
            if (Mode == EditorEditMode.Edit && TemplateRec != null)
                tbName.Value = TemplateRec.TemplateName;

            tbMachineName = new TextBox(31);
            tbMachineName.Anchor = new Anchor(true, true, false, true);
            tbMachineName.Position = new Position(120, 35);
            tbMachineName.Size.Width = 176;
            tbMachineName.Mask = MaskTemplates.MachineName;
            if (Mode == EditorEditMode.Edit && TemplateRec != null)
                tbMachineName.Value = TemplateRec.TemplateMachineName;

            tbCategory = new TextBox(31);
            tbCategory.Anchor = new Anchor(true, true, false, true);
            tbCategory.Position = new Position(120, 60);
            tbCategory.Size.Width = 176;
            if (PreCategory != "")
                tbCategory.Value = PreCategory;
            if (Mode == EditorEditMode.Edit && TemplateRec != null)
                tbCategory.Value = TemplateRec.Category;
        }

        private void P_OnConfirm(DialogConfirmPanel.ConfirmFeedback f)
        {
            if (tbMachineName.Value == "" || tbName.Value == "")
            {
                EditorUtility.DisplayDialog("Invalid name", "Template name or machine name is invalid.\nPlease choose another name.", "OK");
                f.Close = false;
                return;
            }

            if (Mode == EditorEditMode.Create)
            {
                if (!TemplateManager.Create(
                    tbName.Value, tbMachineName.Value,
                    ParentClass, tbCategory.Value, true
                    ))
                    f.Close = false;
            }

            if (Mode == EditorEditMode.Edit)
            {
                if (!TemplateManager.Edit(
                    TemplateRec, tbName.Value, tbMachineName.Value, tbCategory.Value
                    ))
                    f.Close = false;
            }
        }
    }

}